
--
-- Migrate database from schema version 1.9.24.d to 1.9.24.e
--

-- As part of the globalisation work done as part of release 8.5 most string columns
-- are now of type nvarchar.  This meant that some composite key UNIQUE indexes could
-- no longer be created as the maximum data size of the composite key was greater than
-- 900 bytes.  These unique composite indexes have been removed and replace with one
-- or more single-column indexes.  The code does a uniqueness check before inserting.
-- Although there is a race condition this can only occur when two probes have been
-- configured on two separate physical machines monitoring exactly the same thing on
-- a single host.  It does not make sense to set probes up like this and release notes
-- and user documentation will indiciate that this is not a supported configuration

-- As part of the globalisation work in version 8.5 a unique constraint was removed from
-- FLEX_METHOD.  Although the code wasn't handling the INSERT exception that would have 
-- occured it wasn't possible for duplicate rows to be inserted.  Although the stored
-- procedure does a select before checking removing the unique constraing means that there
-- is a race condition that needs to be dealt with.  This stored procedure now uses a 
-- transaction at level SERIALIZABLE in order to ensure that another instance cannot
-- INSERT a row after the initial SELECT has been performed

-- Check that 'd' is the most recent schema upgrade that has been appled
declare @dbVersion nvarchar(20)
declare @requiredVersion nvarchar(20)

set @requiredVersion = '1.9.24.d'
select @dbVersion = IsNull([value], '<unknown>') from db_details where property='schema.version.number' order by ctime desc
	
IF @dbVersion <> @requiredVersion
BEGIN
	PRINT 'This script can only be run against version ' + @requiredVersion + ' of the Results Database.'
	PRINT 'This results database is at version ' + @dbVersion
	PRINT 'You must run previous upgrade scripts in order to upgrade the Results Database to version 1.9.24.d before running this script'
	PRINT 'No changes have been made to the database.'	
	RAISERROR ('Schema is not the correct version for this script',16,-1)        
	set nocount on
    set noexec on
	RETURN
END


-- Before we can change types on columns we need to remove all constraints (PK and UNIQUES) referencing
-- those columns. These will be put back on later, although some of them in slightly different forms
-- where the new index would be too large
PRINT 'REMOVING CONSTRAINTS'

IF (OBJECT_ID('[BWACTIVITY_MRV_U1]') IS NOT NULL) ALTER TABLE [BWACTIVITY_MRV] DROP CONSTRAINT [BWACTIVITY_MRV_U1]
IF (OBJECT_ID('[BWHOST_MRV_U1]') IS NOT NULL) ALTER TABLE [BWHOST_MRV] DROP CONSTRAINT [BWHOST_MRV_U1]
IF (OBJECT_ID('[BWPROC_MRV_U1]') IS NOT NULL) ALTER TABLE [BWPROC_MRV] DROP CONSTRAINT [BWPROC_MRV_U1]
IF (OBJECT_ID('[FLEX_COL_INDEX1]') IS NOT NULL) ALTER TABLE [FLEX_COL] DROP CONSTRAINT [FLEX_COL_INDEX1]
IF (OBJECT_ID('[FLEX_METHOD_INDEX1]') IS NOT NULL) ALTER TABLE [FLEX_METHOD] DROP CONSTRAINT [FLEX_METHOD_INDEX1]
IF (OBJECT_ID('[FLEX_ROW_INDEX1]') IS NOT NULL) ALTER TABLE [FLEX_ROW] DROP CONSTRAINT [FLEX_ROW_INDEX1]
IF (OBJECT_ID('[FLEX_TYPE_INDEX1]') IS NOT NULL) ALTER TABLE [FLEX_TYPE] DROP CONSTRAINT [FLEX_TYPE_INDEX1]
IF (OBJECT_ID('[SONIC_AGENT_MRV_U1]') IS NOT NULL) ALTER TABLE [SONIC_AGENT_MRV] DROP CONSTRAINT [SONIC_AGENT_MRV_U1]
IF (OBJECT_ID('[SONIC_CONNS_MRV_U1]') IS NOT NULL) ALTER TABLE [SONIC_CONNS_MRV] DROP CONSTRAINT [SONIC_CONNS_MRV_U1]
IF (OBJECT_ID('[SONIC_QUEUE_MRV_U1]') IS NOT NULL) ALTER TABLE [SONIC_QUEUE_MRV] DROP CONSTRAINT [SONIC_QUEUE_MRV_U1]
IF (OBJECT_ID('[SYSSTAT_PROC_STATS_MRV_U1]') IS NOT NULL) ALTER TABLE [SYSSTAT_PROC_STATS_MRV] DROP CONSTRAINT [SYSSTAT_PROC_STATS_MRV_U1]
IF (OBJECT_ID('[AGENT_U1]') IS NOT NULL) ALTER TABLE [AGENT] DROP CONSTRAINT [AGENT_U1]
IF (OBJECT_ID('[APPMODELITEM_U1]') IS NOT NULL) ALTER TABLE [APPMODELITEM] DROP CONSTRAINT [APPMODELITEM_U1]
IF (OBJECT_ID('[DQMEMBER_MRV_U1]') IS NOT NULL) ALTER TABLE [DQMEMBER_MRV] DROP CONSTRAINT [DQMEMBER_MRV_U1]
IF (OBJECT_ID('[DQGROUP_MRV_U1]') IS NOT NULL) ALTER TABLE [DQGROUP_MRV] DROP CONSTRAINT [DQGROUP_MRV_U1]
IF (OBJECT_ID('[EMS_QUEUE_MRV_U1]') IS NOT NULL) ALTER TABLE [EMS_QUEUE_MRV] DROP CONSTRAINT [EMS_QUEUE_MRV_U1]
IF (OBJECT_ID('EMS_ROUTE_MRV_U1') IS NOT NULL) ALTER TABLE [EMS_ROUTE_MRV] DROP CONSTRAINT EMS_ROUTE_MRV_U1
IF (OBJECT_ID('[EMS_SERVER_MRV_U1]') IS NOT NULL) ALTER TABLE [EMS_SERVER_MRV] DROP CONSTRAINT [EMS_SERVER_MRV_U1]
IF (OBJECT_ID('[EMS_TOPIC_MRV_U1]') IS NOT NULL) ALTER TABLE [EMS_TOPIC_MRV] DROP CONSTRAINT [EMS_TOPIC_MRV_U1]
IF (OBJECT_ID('[PT_CHARTS_PK]') IS NOT NULL) ALTER TABLE [PERFORM_CHARTS] DROP CONSTRAINT [PT_CHARTS_PK]
IF (OBJECT_ID('[PERSON_USERNAME]') IS NOT NULL) ALTER TABLE [PERSON] DROP CONSTRAINT [PERSON_USERNAME]
IF (OBJECT_ID('[RVHI_MRV_U1]') IS NOT NULL) ALTER TABLE [RVHI_MRV] DROP CONSTRAINT [RVHI_MRV_U1]
IF (OBJECT_ID('[RVTRACE_MULTICAST_MRV_U1]') IS NOT NULL) ALTER TABLE [RVTRACE_MULTICAST_MRV] DROP CONSTRAINT [RVTRACE_MULTICAST_MRV_U1]
IF (OBJECT_ID('[RVTRACE_NETWORK_MRV_U1]') IS NOT NULL) ALTER TABLE [RVTRACE_NETWORK_MRV] DROP CONSTRAINT [RVTRACE_NETWORK_MRV_U1]
IF (OBJECT_ID('[RVTRACE_PTP_MRV_U1]') IS NOT NULL) ALTER TABLE [RVTRACE_PTP_MRV] DROP CONSTRAINT [RVTRACE_PTP_MRV_U1]
IF (OBJECT_ID('[RVTRACE_SUBJECT_MRV_U1]') IS NOT NULL) ALTER TABLE [RVTRACE_SUBJECT_MRV] DROP CONSTRAINT [RVTRACE_SUBJECT_MRV_U1]
IF (OBJECT_ID('[SCENARIO_U1]') IS NOT NULL) ALTER TABLE [SCENARIO] DROP CONSTRAINT [SCENARIO_U1]
IF (OBJECT_ID('[SONIC_BROKER_MRV_U1]') IS NOT NULL) ALTER TABLE [SONIC_BROKER_MRV] DROP CONSTRAINT [SONIC_BROKER_MRV_U1]
IF (OBJECT_ID('[SYSSTAT_SYS_STATS_MRV_U1]') IS NOT NULL) ALTER TABLE [SYSSTAT_SYS_STATS_MRV] DROP CONSTRAINT [SYSSTAT_SYS_STATS_MRV_U1]
IF (OBJECT_ID('TEST_CYCLE_UNIQUE') IS NOT NULL) ALTER TABLE [TEST_CYCLE] DROP CONSTRAINT TEST_CYCLE_UNIQUE
IF (OBJECT_ID('TEST_CYCLE_PROJECT_ENV_PK') IS NOT NULL) ALTER TABLE [TEST_CYCLE_PROJECT_ENV] DROP CONSTRAINT TEST_CYCLE_PROJECT_ENV_PK
IF (OBJECT_ID('[TRANSACTION_U1]') IS NOT NULL) ALTER TABLE [TRANSACTION] DROP CONSTRAINT [TRANSACTION_U1]
IF (OBJECT_ID('[RVTRACE_RETRANS_MRV_U1]') IS NOT NULL) ALTER TABLE [RVTRACE_RETRANS_MRV] DROP CONSTRAINT [RVTRACE_RETRANS_MRV_U1]
IF (OBJECT_ID('[TEST_CYCLE_PROJECT_USR_PK]') IS NOT NULL) ALTER TABLE [TEST_CYCLE_PROJECT_USER] DROP CONSTRAINT TEST_CYCLE_PROJECT_USR_PK
IF (OBJECT_ID('[STATUS_U1]') IS NOT NULL) ALTER TABLE [STATUS] DROP CONSTRAINT [STATUS_U1]

-- Change all varchar to nvarchar and text to nvarchar(max)
PRINT 'CHANGING VARCHARS TO NVARCHARS'

ALTER TABLE [ACTION] ALTER COLUMN [DISPLAY] nvarchar(128)
ALTER TABLE [AGENT] ALTER COLUMN [URL] nvarchar(100) NOT NULL
ALTER TABLE [APPMODELITEM] ALTER COLUMN [DISPLAY] nvarchar(255)
ALTER TABLE [BWACTIVITY_MRV] ALTER COLUMN [ACTIVITYNAME] nvarchar(256) NOT NULL
ALTER TABLE [BWACTIVITY_MRV] ALTER COLUMN [PROCESSNAME] nvarchar(256) NOT NULL
ALTER TABLE [BWACTIVITY_MRV] ALTER COLUMN [ACTIVITY_CLASS] nvarchar(256) NOT NULL
ALTER TABLE [BWHOST_MRV] ALTER COLUMN [HOSTNAME] nvarchar(256) NOT NULL
ALTER TABLE [BWHOST_MRV] ALTER COLUMN [INSTANCENAME] nvarchar(256) NOT NULL
ALTER TABLE [BWHOST_MRV] ALTER COLUMN [IPADDRESS] nvarchar(45) NOT NULL
ALTER TABLE [BWHOST_MRV] ALTER COLUMN [DOMAIN] nvarchar(256) NOT NULL
ALTER TABLE [BWPROC_MRV] ALTER COLUMN [PROCESSNAME] NVARCHAR(256) NOT NULL
ALTER TABLE [BWPROC_MRV] ALTER COLUMN [STARTERNAME] NVARCHAR(256)
ALTER TABLE [COUNTER_DETAILS] ALTER COLUMN [NAME] NVARCHAR(255) NOT NULL
ALTER TABLE [COUNTER_DETAILS] ALTER COLUMN [TYPE] NVARCHAR(20) NOT NULL
ALTER TABLE [COVERAGE] ALTER COLUMN [TYPE] NVARCHAR(10) NOT NULL
ALTER TABLE [DB_DETAILS] ALTER COLUMN [PROPERTY] NVARCHAR(100) NOT NULL
ALTER TABLE [DB_DETAILS] ALTER COLUMN [VALUE] NVARCHAR(100) NOT NULL
ALTER TABLE [DETAIL_EXECUTION] ALTER COLUMN [FAILED_ITERATIONS] NVARCHAR(500)
ALTER TABLE [DETAIL_EXECUTION] ALTER COLUMN [USERNAME] NVARCHAR(50)
ALTER TABLE [DETAIL_EXECUTION] ALTER COLUMN [HOSTNAME] NVARCHAR(100)
ALTER TABLE [DQGROUP_MRV] ALTER COLUMN [DQNAME] NVARCHAR(100) NOT NULL
ALTER TABLE [DQGROUP_MRV] ALTER COLUMN [GRP] NVARCHAR(30) NOT NULL
ALTER TABLE [DQGROUP_MRV] ALTER COLUMN [HOSTIP] NVARCHAR(20) NOT NULL
ALTER TABLE [DQGROUP_MRV] ALTER COLUMN [HOSTNAME] NVARCHAR(50) NOT NULL
ALTER TABLE [DQMEMBER_MRV] ALTER COLUMN [MEMBERTAG] NVARCHAR(20) NOT NULL
ALTER TABLE [DQMEMBER_MRV] ALTER COLUMN [MEMBERID] NVARCHAR(100) NOT NULL
ALTER TABLE [DQMEMBER_MRV] ALTER COLUMN [HOSTIP] NVARCHAR(20) NOT NULL
ALTER TABLE [DQMEMBER_MRV] ALTER COLUMN [HOSTNAME] NVARCHAR(50) NOT NULL
ALTER TABLE [DQMEMBER_MRV] ALTER COLUMN [TYPE] NVARCHAR(10) NOT NULL
ALTER TABLE [DT] ALTER COLUMN [NAME] NVARCHAR(50) NOT NULL
ALTER TABLE [EMS_QUEUE_MRV] ALTER COLUMN [NAME] NVARCHAR(100) NOT NULL
ALTER TABLE [EMS_ROUTE_MRV] ALTER COLUMN [REMOTESERVER] NVARCHAR(100) NOT NULL
ALTER TABLE [EMS_ROUTE_MRV] ALTER COLUMN [REMOTEURL] NVARCHAR(100) NOT NULL
ALTER TABLE [EMS_ROUTE_MRV] ALTER COLUMN [ZONE] NVARCHAR(100) NOT NULL
ALTER TABLE [EMS_ROUTE_MRV] ALTER COLUMN [ZONETYPE] NVARCHAR(20) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [BROKER] NVARCHAR(50) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [STATUS] NVARCHAR(10) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [CFG] NVARCHAR(50) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [STATE] NVARCHAR(20) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [URL] NVARCHAR(20) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [HOSTIP] NVARCHAR(20) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [VER] NVARCHAR(20) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [LIC] NVARCHAR(20) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [FILTTYPE] NVARCHAR(20) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [FILTVAL] NVARCHAR(20)
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [TRACETARG] NVARCHAR(20) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [LOGFILE] NVARCHAR(20)
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [LOGSZ] NVARCHAR(20) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [LOGTRACE] NVARCHAR(100) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [CONTRACE] NVARCHAR(100) NOT NULL
ALTER TABLE [EMS_SERVER_MRV] ALTER COLUMN [STOREDIR] NVARCHAR(40) NOT NULL
ALTER TABLE [EMS_TOPIC_MRV] ALTER COLUMN [NAME] NVARCHAR(100) NOT NULL
ALTER TABLE [EXECUTION_LABEL] ALTER COLUMN [LABEL] NVARCHAR(4000) NOT NULL
ALTER TABLE [EXECUTION_NOTE] ALTER COLUMN [NOTE] NVARCHAR(MAX) NOT NULL
ALTER TABLE [FLEX_COL] ALTER COLUMN [DATA] NVARCHAR(891)
ALTER TABLE [FLEX_METHOD] ALTER COLUMN [DATA] NVARCHAR(891)
ALTER TABLE [FLEX_ROW] ALTER COLUMN [DATA] NVARCHAR(891)
ALTER TABLE [FLEX_TYPE] ALTER COLUMN [DATA] NVARCHAR(891)
ALTER TABLE [LOGMEASUREMENT_MRV] ALTER COLUMN [ATTR_VAL_1] NVARCHAR(255)
ALTER TABLE [LOGMEASUREMENT_MRV] ALTER COLUMN [ATTR_VAL_2] NVARCHAR(255)
ALTER TABLE [LOGMEASUREMENT_MRV] ALTER COLUMN [ATTR_VAL_3] NVARCHAR(255)
ALTER TABLE [LOGMEASUREMENT_MRV] ALTER COLUMN [ATTR_VAL_4] NVARCHAR(255)
ALTER TABLE [LOGMEASUREMENT_MRV] ALTER COLUMN [ATTR_VAL_5] NVARCHAR(255)
ALTER TABLE [LOGMEASUREMENT_TS] ALTER COLUMN [STRCTR_VAL_1] NVARCHAR(255)
ALTER TABLE [LOGMEASUREMENT_TS] ALTER COLUMN [STRCTR_VAL_2] NVARCHAR(255)
ALTER TABLE [LOGMEASUREMENT_TS] ALTER COLUMN [STRCTR_VAL_3] NVARCHAR(255)
ALTER TABLE [LOGMEASUREMENT_TS] ALTER COLUMN [STRCTR_VAL_4] NVARCHAR(255)
ALTER TABLE [PERFORM_CHARTS] ALTER COLUMN [CHART_NAME] NVARCHAR(100) NOT NULL
ALTER TABLE [PERFORM_CHARTS] ALTER COLUMN [DATA] NVARCHAR(MAX) NOT NULL
ALTER TABLE [PERSON] ALTER COLUMN [USERNAME] NVARCHAR(32) NOT NULL
ALTER TABLE [PROBE_CONFIG] ALTER COLUMN [CONFIG] NVARCHAR(MAX)
ALTER TABLE [PROJECT] ALTER COLUMN [NAME] NVARCHAR(255) NOT NULL
ALTER TABLE [RVHI_MRV] ALTER COLUMN [GRP] NVARCHAR(30) NOT NULL
ALTER TABLE [RVHI_MRV] ALTER COLUMN [HOSTIP] NVARCHAR(20) NOT NULL
ALTER TABLE [RVHI_MRV] ALTER COLUMN [HOSTNAME] NVARCHAR(50) NOT NULL
ALTER TABLE [RVHI_MRV] ALTER COLUMN [NETWORK] NVARCHAR(20) NOT NULL
ALTER TABLE [RVHI_MRV] ALTER COLUMN [OS] NVARCHAR(20) NOT NULL
ALTER TABLE [RVHI_MRV] ALTER COLUMN [VER] NVARCHAR(10) NOT NULL
ALTER TABLE [RVHI_MRV] ALTER COLUMN [HTTPADDR] NVARCHAR(20) NOT NULL
ALTER TABLE [RVHI_MRV] ALTER COLUMN [USERNAME] NVARCHAR(30) NOT NULL
ALTER TABLE [RVHI_MRV] ALTER COLUMN [SERVICE] NVARCHAR(20) NOT NULL
ALTER TABLE [RVTRACE_MULTICAST_MRV] ALTER COLUMN [SERVICE] NVARCHAR(20) NOT NULL
ALTER TABLE [RVTRACE_MULTICAST_MRV] ALTER COLUMN [HOSTIP] NVARCHAR(20) NOT NULL
ALTER TABLE [RVTRACE_MULTICAST_MRV] ALTER COLUMN [HOSTNAME] NVARCHAR(50) NOT NULL
ALTER TABLE [RVTRACE_MULTICAST_MRV] ALTER COLUMN [DESTINATION] NVARCHAR(20) NOT NULL
ALTER TABLE [RVTRACE_NETWORK_MRV] ALTER COLUMN [NETWORKID] NVARCHAR(20) NOT NULL
ALTER TABLE [RVTRACE_PTP_MRV] ALTER COLUMN [SERVICE] NVARCHAR(20) NOT NULL
ALTER TABLE [RVTRACE_PTP_MRV] ALTER COLUMN [HOSTIP] NVARCHAR(20) NOT NULL
ALTER TABLE [RVTRACE_PTP_MRV] ALTER COLUMN [HOSTNAME] NVARCHAR(50) NOT NULL
ALTER TABLE [RVTRACE_RETRANS_MRV] ALTER COLUMN [SERVICE] NVARCHAR(20) NOT NULL
ALTER TABLE [RVTRACE_RETRANS_MRV] ALTER COLUMN [HOSTIP] NVARCHAR(20) NOT NULL
ALTER TABLE [RVTRACE_RETRANS_MRV] ALTER COLUMN [HOSTNAME] NVARCHAR(50) NOT NULL
ALTER TABLE [RVTRACE_SUBJECT_MRV] ALTER COLUMN [SERVICE] NVARCHAR(20) NOT NULL
ALTER TABLE [RVTRACE_SUBJECT_MRV] ALTER COLUMN [SUBJECT] NVARCHAR(150) NOT NULL
ALTER TABLE [SCENARIO] ALTER COLUMN [DISPLAY] NVARCHAR(64)
ALTER TABLE [SONIC_AGENT_MRV] ALTER COLUMN [DOMAIN] NVARCHAR(256) NOT NULL
ALTER TABLE [SONIC_AGENT_MRV] ALTER COLUMN [CONTAINER] NVARCHAR(256) NOT NULL
ALTER TABLE [SONIC_BROKER_MRV] ALTER COLUMN [BROKER] NVARCHAR(256) NOT NULL
ALTER TABLE [SONIC_CONNS_MRV] ALTER COLUMN [BROKER] NVARCHAR(256) NOT NULL
ALTER TABLE [SONIC_CONNS_MRV] ALTER COLUMN [CONN] NVARCHAR(256) NOT NULL
ALTER TABLE [SONIC_QUEUE_MRV] ALTER COLUMN [BROKER] NVARCHAR(256) NOT NULL
ALTER TABLE [SONIC_QUEUE_MRV] ALTER COLUMN [QUEUE] NVARCHAR(256) NOT NULL
ALTER TABLE [STATUS] ALTER COLUMN [DISPLAY] NVARCHAR(10) NOT NULL
ALTER TABLE [SYSSTAT_PROC_STATS_MRV] ALTER COLUMN [PROCESS_NAME] NVARCHAR(256) NOT NULL
ALTER TABLE [SYSSTAT_PROC_STATS_MRV] ALTER COLUMN [PROCESS_ID] NVARCHAR(256) NOT NULL
ALTER TABLE [SYSSTAT_SYS_STATS_MRV] ALTER COLUMN [HOSTNAME] NVARCHAR(256) NOT NULL
ALTER TABLE [TESTENGINE_ENGINE_MRV] ALTER COLUMN [URL] NVARCHAR(255) NOT NULL
ALTER TABLE [TESTENGINE_SEQ_MRV] ALTER COLUMN [SEQ_NAME] NVARCHAR(255) NOT NULL
ALTER TABLE [TESTENGINE_TRANS_MRV] ALTER COLUMN [TRANS_NAME] NVARCHAR(255) NOT NULL
ALTER TABLE [TESTER_SEQ_END_MRV] ALTER COLUMN [SEQ_NAME] NVARCHAR(255) NOT NULL
ALTER TABLE [TESTER_SEQ_MRV] ALTER COLUMN [SEQ_NAME] NVARCHAR(255) NOT NULL
ALTER TABLE [TESTER_TRANS_END_MRV] ALTER COLUMN [TRANS_NAME] NVARCHAR(255) NOT NULL
ALTER TABLE [TESTER_TRANS_MRV] ALTER COLUMN [TRANS_NAME] NVARCHAR(255) NOT NULL
ALTER TABLE [TEST_CYCLE] ALTER COLUMN [DISPLAY] NVARCHAR(256) NOT NULL
ALTER TABLE [TEST_CYCLE] ALTER COLUMN [COMMENTS] NVARCHAR(4000)
ALTER TABLE [TEST_CYCLE] ALTER COLUMN [CREATED_BY] NVARCHAR(256) NOT NULL
ALTER TABLE [TEST_CYCLE] ALTER COLUMN [CLOSED_BY] NVARCHAR(256)
ALTER TABLE [TEST_CYCLE_PROJECT_ENV] ALTER COLUMN [ENVIRONMENT] NVARCHAR(256) NOT NULL
ALTER TABLE [TEST_CYCLE_PROJECT_USER] ALTER COLUMN [USERNAME] NVARCHAR(256) NOT NULL
ALTER TABLE [TRANSACTION] ALTER COLUMN [NAME] NVARCHAR(255) NOT NULL
ALTER TABLE [WMBROKER_B_MRV] ALTER COLUMN [HOST] NVARCHAR(256) NOT NULL
ALTER TABLE [WMBROKER_B_MRV] ALTER COLUMN [BROKER] NVARCHAR(256) NOT NULL
ALTER TABLE [WMBROKER_CG_MRV] ALTER COLUMN [CLIENT_GROUP] NVARCHAR(256) NOT NULL
ALTER TABLE [WMBROKER_C_MRV] ALTER COLUMN [CLIENT] NVARCHAR(256) NOT NULL
ALTER TABLE [WMBROKER_ET_MRV] ALTER COLUMN [EVENT_TYPE] NVARCHAR(256) NOT NULL
ALTER TABLE [WMIS_SERVER_MRV] ALTER COLUMN [HOST] NVARCHAR(256) NOT NULL
ALTER TABLE [WMIS_SERVICE_MRV] ALTER COLUMN [SERVICE] NVARCHAR(256) NOT NULL

-- Now put all the constraints back again (that we had to remove to change the columns)
-- We can't put UNIQUE constraints that would now be > 900 bytes back on - those
-- are changed to multiple indexes later on
PRINT 'RESTORING CONSTRAINTS'

 ALTER TABLE [DQGROUP_MRV] ADD CONSTRAINT [DQGROUP_MRV_U1] UNIQUE
 (
 [DQNAME],
 [GRP]
 )

ALTER TABLE [DQMEMBER_MRV] ADD  CONSTRAINT [DQMEMBER_MRV_U1] UNIQUE
 (
 [DQGROUP_MRV_ID],
 [MEMBERTAG]
 )

  ALTER TABLE [EMS_QUEUE_MRV] ADD CONSTRAINT [EMS_QUEUE_MRV_U1] UNIQUE
 (
 [EMS_SERVER_MRV_ID],
 [NAME]
 )

ALTER TABLE [EMS_ROUTE_MRV] ADD CONSTRAINT [EMS_ROUTE_MRV_U1] UNIQUE
 (
 [EMS_SERVER_MRV_ID],
 [REMOTESERVER]
 )

 ALTER TABLE [EMS_SERVER_MRV] ADD CONSTRAINT [EMS_SERVER_MRV_U1] UNIQUE
 (
 [BROKER]
 )

 ALTER TABLE [EMS_TOPIC_MRV] ADD  CONSTRAINT [EMS_TOPIC_MRV_U1] UNIQUE
 (
 [EMS_SERVER_MRV_ID],
 [NAME]
 )

 ALTER TABLE [PERFORM_CHARTS] ADD CONSTRAINT [PT_CHARTS_PK] PRIMARY KEY
 (
 [CHART_NAME],
 [PROJECT_ID]
 )
 
 ALTER TABLE [AGENT] ADD CONSTRAINT [AGENT_U1] UNIQUE
 (
 [URL]
 )

 ALTER TABLE [APPMODELITEM] ADD CONSTRAINT [APPMODELITEM_U1] UNIQUE
 (
 [PROJECT_ID],
 [UNIQUEID]
 )

 ALTER TABLE [BWPROC_MRV] ADD CONSTRAINT [BWPROC_MRV_U1] UNIQUE
 (
 [BWHOST_MRV_ID],
 [PROCESSNAME]
 )

 ALTER TABLE [PERSON] ADD  CONSTRAINT [PERSON_USERNAME] UNIQUE
 (
 [USERNAME]
 )

 ALTER TABLE [RVHI_MRV] ADD CONSTRAINT [RVHI_MRV_U1] UNIQUE
 (
 [GRP],
 [HOSTIP],
 [HOSTNAME],
 [SERVICE]
 )

 ALTER TABLE [RVTRACE_MULTICAST_MRV] ADD  CONSTRAINT [RVTRACE_MULTICAST_MRV_U1] UNIQUE
 (
 [RVTRACE_NETWORK_MRV_ID],
 [SERVICE],
 [HOSTIP],
 [HOSTNAME],
 [DESTINATION]
 )

 ALTER TABLE [RVTRACE_NETWORK_MRV] ADD  CONSTRAINT [RVTRACE_NETWORK_MRV_U1] UNIQUE
 (
 [NETWORKID]
 )

 ALTER TABLE [RVTRACE_PTP_MRV] ADD  CONSTRAINT [RVTRACE_PTP_MRV_U1] UNIQUE
 (
 [RVTRACE_NETWORK_MRV_ID],
 [SERVICE],
 [HOSTIP],
 [HOSTNAME]
 )

 ALTER TABLE [RVTRACE_SUBJECT_MRV] ADD  CONSTRAINT [RVTRACE_SUBJECT_MRV_U1] UNIQUE
 (
 [RVTRACE_NETWORK_MRV_ID],
 [SERVICE],
 [SUBJECT]
 )

 ALTER TABLE [SCENARIO] ADD  CONSTRAINT [SCENARIO_U1] UNIQUE
 (
 [APPMODELITEM_ID],
 [UNIQUEID],
 [DISPLAY]
 )

 ALTER TABLE [SONIC_BROKER_MRV] ADD  CONSTRAINT [SONIC_BROKER_MRV_U1] UNIQUE
 (
 [SONIC_AGENT_MRV_ID],
 [BROKER]
 )

 ALTER TABLE [SYSSTAT_SYS_STATS_MRV] ADD  CONSTRAINT [SYSSTAT_SYS_STATS_MRV_U1] UNIQUE
 (
 [HOSTNAME]
 )

 ALTER TABLE [TEST_CYCLE] ADD  CONSTRAINT [TEST_CYCLE_UNIQUE] UNIQUE
 (
 [DISPLAY]
 )

 ALTER TABLE [TEST_CYCLE_PROJECT_ENV] ADD  CONSTRAINT [TEST_CYCLE_PROJECT_ENV_PK] PRIMARY KEY
 (
 [TEST_CYCLE_PROJECT_ID],
 [ENVIRONMENT]
 )

 ALTER TABLE [TRANSACTION] ADD  CONSTRAINT [TRANSACTION_U1] UNIQUE
 (
 [NAME]
 )

ALTER TABLE [RVTRACE_RETRANS_MRV] ADD  CONSTRAINT [RVTRACE_RETRANS_MRV_U1] UNIQUE
 (
 [RVTRACE_NETWORK_MRV_ID],
 [SERVICE],
 [HOSTIP],
 [HOSTNAME]
 )

ALTER TABLE [TEST_CYCLE_PROJECT_USER] ADD  CONSTRAINT [TEST_CYCLE_PROJECT_USR_PK] PRIMARY KEY
 (
 [TEST_CYCLE_PROJECT_ID],
 [USERNAME]
 )


ALTER TABLE [STATUS] ADD CONSTRAINT [STATUS_U1] UNIQUE
 (
 [DISPLAY]
 ) 


 -- Add some new indexes to make up for the multi-column unique ones
PRINT 'CREATING NEW INDEXES'

CREATE INDEX [BWACTIVITY_MRV_IDX1] ON [BWACTIVITY_MRV]  
(
 [BWHOST_MRV_ID]
)

CREATE INDEX [BWACTIVITY_MRV_IDX2] ON [BWACTIVITY_MRV]  
(
 [ACTIVITYNAME]
)

CREATE INDEX [BWACTIVITY_MRV_IDX3] ON [BWACTIVITY_MRV]  
(
 [PROCESSNAME]
)

CREATE INDEX [BWHOST_MRV_IDX] ON [BWHOST_MRV]  
(
 [INSTANCENAME]
)

CREATE INDEX [FLEX_COL_IDX1] ON [FLEX_COL]
(
 [METHOD]
)

CREATE INDEX [FLEX_METHOD_IDX1] ON [FLEX_METHOD]
 (
 [TYPE]
 )

CREATE INDEX [FLEX_ROW_IDX1] ON [FLEX_ROW]
 (
 [COL]
 )
 
CREATE INDEX [SONIC_AGENT_MRX_IDX1] ON [SONIC_AGENT_MRV]
 (
 [DOMAIN]
 )
 
CREATE INDEX [SONIC_AGENT_MRX_IDX2] ON [SONIC_AGENT_MRV]
 (
 [CONTAINER]
 )
 
CREATE INDEX [SONIC_CONNS_MRV_IDX1] ON [SONIC_CONNS_MRV]
 (
 [SONIC_AGENT_MRV_ID],
 [BROKER]
 )
 
CREATE INDEX [SONIC_CONNS_MRV_IDX2] ON [SONIC_CONNS_MRV]
 (
 [CONN]
 )
 
CREATE INDEX [SONIC_QUEUE_MRV_IDX1] ON [SONIC_QUEUE_MRV]
 (
 [SONIC_AGENT_MRV_ID],
 [BROKER]
 )

CREATE INDEX [SONIC_QUEUE_MRV_IDX2] ON [SONIC_QUEUE_MRV]
 (
 [QUEUE]
 )

CREATE INDEX [SYSSTAT_PROC_STATS_MRV_IDX1] ON [SYSSTAT_PROC_STATS_MRV]
 (
 [SYSSTAT_SYS_STATS_MRV_ID],
 [PROCESS_NAME]
 )
 
CREATE INDEX [SYSSTAT_PROC_STATS_MRV_IDX2] ON [SYSSTAT_PROC_STATS_MRV]
 (
 [PROCESS_ID]
 )

GO


PRINT 'DROPPING PROCEDURES AND VIEWS'
GO
IF (OBJECT_ID('INSERT_FLEX_COL') IS NOT NULL) DROP PROCEDURE INSERT_FLEX_COL
IF (OBJECT_ID('INSERT_FLEX_METHOD') IS NOT NULL) DROP PROCEDURE INSERT_FLEX_METHOD
IF (OBJECT_ID('INSERT_FLEX_ROW') IS NOT NULL) DROP PROCEDURE INSERT_FLEX_ROW
IF (OBJECT_ID('INSERT_FLEX_TYPE') IS NOT NULL) DROP PROCEDURE INSERT_FLEX_TYPE
IF (OBJECT_ID('[SCENARIO_EXECUTIONS]') IS NOT NULL) DROP VIEW [SCENARIO_EXECUTIONS]
IF (OBJECT_ID('[TESTER_PERF_MRV]') IS NOT NULL) DROP VIEW [TESTER_PERF_MRV]
GO

-- THESE WILL FAIL IF THE DROPS DIDN'T WORK
PRINT 'CREATING NEW STORED PROCEDURES AND VIEWS'
GO

CREATE PROCEDURE INSERT_FLEX_COL (
 @PK [numeric] OUTPUT,
 @METHOD [numeric],
 @DATA [nvarchar] (891)
) AS
BEGIN

	-- In the majority of cases there will be a PK already.
	-- Do a quick attempt to get it before locking tables

	SELECT @PK = PK FROM FLEX_COL WHERE @METHOD = METHOD
									AND @DATA   = DATA

	if @PK IS NULL
	BEGIN
	  -- We're the first.  Need to protect against race conditions now
	  SET TRANSACTION ISOLATION LEVEL SERIALIZABLE

	  BEGIN TRANSACTION

	  -- This statement will effectively lock the table so only one thread will get NULL
	  SELECT @PK = PK FROM FLEX_COL WHERE @METHOD = METHOD
									  AND @DATA   = DATA

	  -- The SERIALIZABLE transaction level means no other threads can modify the
	  -- underlying table.
	  IF @PK IS NULL
	  BEGIN
		 INSERT
		   INTO FLEX_COL
		  ( METHOD, DATA
		  ) VALUES
		  ( @METHOD, @DATA
		  )
		  SELECT @PK = SCOPE_IDENTITY()
	  END

	  COMMIT TRANSACTION
	  END
	END
GO

-- See comments for procedure CREATE PROCEDURE INSERT_FLEX_COL regarding why this
-- is implemented in this way
CREATE PROCEDURE INSERT_FLEX_METHOD (
 @PK [numeric] OUTPUT,
 @TYPE [numeric],
 @DATA [nvarchar] (891)
) AS
BEGIN

	SELECT @PK = PK FROM FLEX_METHOD WHERE @TYPE = TYPE
									   AND @DATA = DATA

	IF @PK IS NULL
	BEGIN

		SET TRANSACTION ISOLATION LEVEL SERIALIZABLE

		BEGIN TRANSACTION

		SELECT @PK = PK FROM FLEX_METHOD WHERE @TYPE = TYPE
										   AND @DATA = DATA
		IF @PK IS NULL
		BEGIN
		   INSERT
			 INTO FLEX_METHOD
			( TYPE, DATA
			) VALUES
			( @TYPE, @DATA
			)
			SELECT @PK = SCOPE_IDENTITY()
		END

		COMMIT TRANSACTION
	END
END
GO

-- See comments for procedure CREATE PROCEDURE INSERT_FLEX_COL regarding why this
-- is implemented in this way
CREATE PROCEDURE INSERT_FLEX_ROW (
 @PK [numeric] OUTPUT,
 @COL [numeric],
 @DATA [nvarchar] (891)
) AS
BEGIN

	SELECT @PK = PK FROM FLEX_ROW WHERE @COL  = COL
									AND @DATA = DATA

	IF @PK IS NULL
	BEGIN

		SET TRANSACTION ISOLATION LEVEL SERIALIZABLE

		BEGIN TRANSACTION

		SELECT @PK = PK FROM FLEX_ROW WHERE @COL  = COL
										AND @DATA = DATA
		IF @PK IS NULL
		BEGIN
		   INSERT
			 INTO FLEX_ROW
			( COL, DATA
			) VALUES
			( @COL, @DATA
			)
			SELECT @PK = SCOPE_IDENTITY()
		END

		COMMIT TRANSACTION
	END
END
GO

-- See comments for procedure CREATE PROCEDURE INSERT_FLEX_COL regarding why this
-- is implemented in this way
CREATE PROCEDURE INSERT_FLEX_TYPE (
 @PK [numeric] OUTPUT,
 @DATA [nvarchar] (891)
) AS
BEGIN

	SELECT @PK = PK FROM FLEX_TYPE WHERE @DATA = DATA

	IF @PK IS NULL
	BEGIN
		SET TRANSACTION ISOLATION LEVEL SERIALIZABLE

		BEGIN TRANSACTION

		SELECT @PK = PK FROM FLEX_TYPE WHERE @DATA = DATA
		IF @PK IS NULL
		BEGIN
		   INSERT
			 INTO FLEX_TYPE
			( DATA
			) VALUES
			( @DATA
			)
			SELECT @PK = SCOPE_IDENTITY()
		END

		COMMIT TRANSACTION
	END
END
GO

CREATE VIEW [SCENARIO_EXECUTIONS] (
 [EXECUTION_ID],
 [LABEL],
 [LOCKBY],
 [LOCKDATE],
 [NOTES_PK],
 [PROJECT_PK],
 [SUITE_PK],
 [SUITE_ID],
 [SUITE_NAME],
 [SCENARIO_NAME],
 [SCENARIO_ID],
 [START_TIME],
 [END_TIME],
 [STATUS],
 [EXPIRES],
 [ROOT_EXEC_ID]
) AS 
  SELECT sce.execution_id, l.label, u.username, u.time, n.execution_id, t.project_id, t.id, t.uniqueid, t.display, s.display, s.uniqueid, e.start_time, e.end_time, (SELECT display FROM status WHERE id = e.status_id), t.expiration, e.root_id
  FROM scenario s, scenario_execution sce, execution e
  LEFT OUTER JOIN execution_label l ON e.id = l.execution_id
  LEFT OUTER JOIN (
    SELECT sue.execution_id, app.id, app.uniqueid, app.display, app.project_id, sue.expiration
    FROM suite_execution sue, appmodelitem app
    WHERE sue.appmodelitem_id = app.id
  ) t ON e.id = t.execution_id
  LEFT OUTER JOIN (
    SELECT el.execution_id, p.username, el.time
    FROM execution_lock el, person p
    WHERE el.person_id = p.id
  ) u ON e.id = u.execution_id
  LEFT OUTER JOIN execution_note n ON e.id = n.execution_id
  WHERE e.id = sce.execution_id AND sce.scenario_id = s.id
   
GO

CREATE VIEW [TESTER_PERF_MRV] (
 [ID],
 [NAME],
 [GUID],
 [TIME]
) AS 
select a.ID, a.display, p.uniqueid as guid, 0 as time
from appmodelitem a, project p
where a.project_id = p.id
GO

IF NOT EXISTS (SELECT  * FROM DB_DETAILS WHERE PROPERTY='schema.version.number' and VALUE = '1.9.24.e')
BEGIN
PRINT 'INCREMENTING SCHEMA VERSION'

BEGIN TRANSACTION
INSERT INTO db_details VALUES
 (GETDATE(), 'schema.version.number', '1.9.24.e')
COMMIT TRANSACTION
END

GO

set nocount off
set noexec off
	
